@@ -240,6 +240,8 @@ typedef struct lang_input_statement_struct
 
   bfd *the_bfd;
 
+  struct flag_info *section_flag_list;
+
   /* Point to the next file - whatever it is, wanders up and down
      archives */
   union lang_statement_union *next;
@@ -279,9 +281,6 @@ typedef struct lang_input_statement_struct
   /* Whether to include the entire contents of an archive.  */
   unsigned int whole_archive : 1;
 
-  /* Set if the file has been processed in stage 1 .  */
-  unsigned int stage1 : 1;
-
   /* Set when bfd opening is successful.  */
   unsigned int loaded : 1;
 
@@ -290,9 +289,14 @@ typedef struct lang_input_statement_struct
   /* Set if the file does not exist.  */
   unsigned int missing_file : 1;
 
+#ifdef ENABLE_PLUGINS
   /* Set if the file was claimed by a plugin.  */
   unsigned int claimed : 1;
 
+  /* Set if the file was claimed from an archive.  */
+  unsigned int claim_archive : 1;
+#endif /* ENABLE_PLUGINS */
+
 } lang_input_statement_type;
 
 typedef struct
@@ -335,6 +339,7 @@ struct lang_wild_statement_struct
   walk_wild_section_handler_t walk_wild_section_handler;
   struct wildcard_list *handler_data[4];
   lang_section_bst_type *tree;
+  struct flag_info *section_flag_list;
 };
 
 typedef struct lang_address_statement_struct
@@ -541,7 +546,7 @@ extern void lang_for_each_file
 extern void lang_reset_memory_regions
   (void);
 extern void lang_do_assignments
-  (void);
+  (lang_phase_type);
 
 #define LANG_FOR_EACH_INPUT_STATEMENT(statement)			\
   lang_input_statement_type *statement;					\
@@ -601,14 +606,10 @@ extern void one_lang_size_sections_pass
   (bfd_boolean *, bfd_boolean);
 extern void lang_add_insert
   (const char *, int);
-extern void lang_enter_input
-  (void);
-extern void lang_leave_input
-  (void);
 extern void lang_enter_group
-  (bfd_boolean);
+  (void);
 extern void lang_leave_group
-  (bfd_boolean);
+  (void);
 extern void lang_add_section
   (lang_statement_list_type *, asection *,
    lang_output_section_statement_type *);
@@ -662,10 +663,7 @@ lang_ld_feature (char *);
 
 typedef enum
 {
-  cmdline_is_lang_input_statement_enum,
   cmdline_is_file_enum,
-  cmdline_is_enter_group_enum,
-  cmdline_is_leave_group_enum,
   cmdline_is_bfd_enum
 } cmdline_enum_type;
 
@@ -675,12 +673,6 @@ typedef struct cmdline_header_struct
   cmdline_enum_type type;
 } cmdline_header_type;
 
-typedef struct cmdline_input_statement_struct
-{
-  cmdline_header_type header;
-  lang_input_statement_type *input;
-} cmdline_input_statement_type;
-
 typedef struct cmdline_file_struct
 {
   cmdline_header_type header;
@@ -696,7 +688,6 @@ typedef struct cmdline_bfd_struct
 typedef union cmdline_union
 {
   cmdline_header_type header;
-  cmdline_input_statement_type input_statement;
   cmdline_file_type file;
   cmdline_bfd_type abfd;
 } cmdline_union_type;
@@ -707,12 +698,6 @@ typedef struct cmdline_list
   cmdline_union_type **tail;
 } cmdline_list_type;
 
-extern unsigned int cmdline_linker_script;
-extern void cmdline_option_append (cmdline_enum_type, void *);
-extern void cmdline_option_append_lang_input (lang_input_statement_type *);
-extern void cmdline_option_insert_claimed_output (const char *);
-extern void cmdline_set_next_claimed_output (bfd *);
-extern void cmdline_claimed_archive_member_list_append (bfd *);
 extern void cmdline_emit_object_only_section (void);
 extern void cmdline_check_object_only_section (bfd *, bfd_boolean);
 extern void cmdline_remove_object_only_files (void);
